##################################################################
# GHOTI Gedcom Html Output Translation Interface.
# Copyright (c) 2006 Christopher Robert Squires
# All rights reserved.
##################################################################
# {{{1
# (Sorry about the legal stuff, I just want to minimise the likelyhood of
# commercial interests profitting from my work without some recognition.)
#
# ANY commercial use of this code is prohibited without written permission.
#
# However, permission is hereby granted to private users of GHOTI to
# make copies and modifications to this code for their own personal
# use and to distribute such modified copies, provided that all three of
# the following conditions are met:
# 1 this copyright notice is retained on all modified or distributed copies,
# 2 and that the changes are record in the revision history, 
# 3 and a copy of the modified code is forwarded to the author.
#
# If I consider that a modification is suitable for inclusion, I will add it
# to the master source, and add details to the revision history.
#
# If you modify the code, please add your name here, with a brief description
# of the modification.
#
# REVISION HISTORY 
#------------------------------------------------------------
# http://www.squiresfamily.me.uk/HowToContactMe.html - 23 May 2006
# Initial release of the open source PERL
#------------------------------------------------------------
# }}}1 

use ghotiModule;
use File::Copy; 

##################################################################
# Grab program argument.
# and configuration data
###################################################################
# {{{1


if (defined( $ARGV[0] ) ) {
	$configFile = $ARGV[0];
        }
    else {
	$configFile = "ghotiSkeletonConfig.gcf";
        }

unless (open (CONFIGFILE,"<".$configFile)){die "\nERROR - configuration not found\n"}

$privateCount = 0;  # Number of individuals forced to private status for personal reasons.

# defaults values
$minPersonsPerIndexPage = 5;

# source file
$cssFileName            = "ghoti\.css";
$readmeFileName         = "readme\.html";

#target file 
$indexFileName          = "index\.html";

#output file for link analysis.
$ghotiLinkReport	= "ghotiLinkReport\.txt";

labelGetConfigLoop:
while ($line=<CONFIGFILE>) {
  chop $line;

  # a "#" causes the text after it to be ignored.
  if ($line =~ /#/) {($commentFree) = ($line =~ /\s*([^#]*)\s*#/); $line = $commentFree;}
  

  if ($line =~ /footnoteStart/) {
	$footnoteLine = ghotiModule::getHTML(\*CONFIGFILE,"footnoteEnd");
	next labelGetConfigLoop;
	}

  if ($line =~ /disableFilter/) { print "WARNING - filter dissabled\n";$disableFilter = 1;}

  elsif ($line =~ /readableHTML/) {
        print "-W- ReadableHTML set\n";
        $optionalNewLine = "\n";
        $optionalTab = "    ";
        $optionalNlTab = "\n    ";
	}
  elsif ($line =~ /suppressSources/) {
        print "-I- Generation of Source Page is suppressed\n";
	$suppressSources = 1;
	}
  elsif ($line =~ /suppressFamilyPages/) {
        print "-W- Generation of Family Pages is suppressed\n";
	$suppressFamilyPages = 1;
	}
  elsif ($line =~ /supressIndexGen/) {
        print "-W- index generaton is suppressed\n";
	$supressIndexGen = 1;
	}
  elsif ($line =~ /suppressExistingLinkChecks/) {
        print "-W- html link checks are suppressed\n";
	$suppressExistingLinkChecks = 1;
	}


  ($settingName) = ($line =~ /^\s*(\S+)\s*=/);

  ($restOfLine) = ($' =~/\s*(\S*)\s*/);
  if ($settingName eq "targetDir")                 { $targetDir      = $restOfLine;}
  elsif ($settingName eq "masterDir")              { $masterDir      = $restOfLine;}
  elsif ($settingName eq "htmlRoot")               { $htmlRoot       = $restOfLine;}
  elsif ($settingName eq "bigPicRoot")             { $bigPicRoot     = $restOfLine;}
  elsif ($settingName eq "smallPicRoot")           { $smallPicRoot   = $restOfLine;}
  elsif ($settingName eq "sourceGed")              { $sourceGed      = $restOfLine;}
  elsif ($settingName eq "gedGeneratedBy")         { $gedGeneratedBy = $restOfLine;}
  elsif ($settingName eq "cssFileName")            { $cssFileName    = $restOfLine;}
  elsif ($settingName eq "readmeFileName")         { $readmeFileName = $restOfLine;}
  elsif ($settingName eq "indexFileName")          { $indexFileName  = $restOfLine;}
  elsif ($settingName eq "linkReportFileName")     { $ghotiLinkReport = $restOfLine;}
  elsif ($settingName eq "minPersonsPerIndexPage") { ($minPersonsPerIndexPage) = ( $restOfLine =~ /(.\d+)/);} 
  elsif ($settingName eq "assertPrivate")          { 
	  ($private[$privateCount]) = ( $restOfLine =~ /(.\d+)/);
	  $privatePerson{$private[$privateCount]} = "Blocking";
	  $privateCount++;}
  elsif ($settingName eq "minPersonsPerIndexPage") { ($minPersonsPerIndexPage) = ( $restOfLine =~ /(.\d+)/);} 
    }

#}}}1	



###################################################################
# Set up the directory structure.
###################################################################
#{{{1

    

if ( -e ($sourceGed.".ged") ) {  # -e = if doesn't exist
        print "-I- Reading data from file ",$sourceGed,".ged\n";
        print "-I- Writing data to the ",$targetDir," directory\n";
        }
  else  {
        die "\nERROR file ".$sourceGed.".ged not found\n";
	}

unless (open (FH,"<".$sourceGed.".ged")) {
	  die "\nERROR - unable to open " . $sourceGed . ".ged for reading";
	}

unless ( -d ($targetDir) ) { mkdir $targetDir, 488 ; }

unless (-e ($targetDir."/".$cssFileName)){copy ("ghotiSkeletonCss\.css",  ($targetDir."/".$cssFileName)) }
unless (-e ($targetDir."/".$readmeFileName)){
	copy ("ghotiSkeletonReadme\.html",  ($targetDir."/".$readmeFileName));
	print "-I- file ".$targetDir."/".$readmeFileName." was not found "
	    . "- ghoti has created a skelton file as a starting point for you";
	}


unless (-e ($targetDir."/".$indexFileName)){copy ("ghotiSkeletonIndex\.html",  ($targetDir."/".$indexFileName)) }


$familyDir = $targetDir."/families";

unless ( -d ($familyDir) ) { mkdir $familyDir, 488; }

      # }}}1

###################################################################
# Grab the data from the Gedcom file.
###################################################################

while ($line=<FH>) {
# {{{1 
# Read and split the line into tag level, identifier and "the rest of the line"
# {{{2
  chop $line;
  ($level, $tag) = ($line =~ /^\s*(\S+)\s+(\S+)/);
  $restOfLine = $';

  #grab the level 0 record identity. 
  #    If ident is /I\S*/ the record is for an individual
  #    If ident is /F\S*/ the record is for a family
  #    If ident is /NI\S*/ the record is for a note about an individual
  #    If ident is /NF\S*/ the record is for a note about a family
  #    If ident is /BI\S*/ the record is for a note about individuals birth (not implemented)
  #    If ident is /CI\S*/ the record is for a note about individuals christening (not implemented)
  #    If ident is /DI\S*/ the record is for a note about individuals death (not implemented)
  #    If ident is /S\S*/ the record is for a Source (not implemented)
  
  if ($level eq "0" && ( ($ident) = ($tag =~ /@(\S+)@/))) {  }
  #}}}2
  

#################################################################################
# Grab Data from individual Record
#################################################################################
# {{{2
  #if ($ident =~ /\WI\S*/ and $ident !~ /[CBDN]I/) { 
  if ($ident =~ /^I\S*/) { 
	if ($ident =~ /\wI/) {print "Diag $ident \n";}

	$ofIndividual = $ident;
        if ($known{$ofIndividual} != 1) { 
                $givenName{$ofIndividual}     = "____";
                $surName{$ofIndividual}       = "____";
		$noOfSpouses{$ofIndividual}   = 0;
		$noOfSources{$ofIndividual}   = 0;
		$noOfOcupation{$ofIndividual} = 0;
		$noOfCensus{$ofIndividual}    = 0;
		$noOfAddress{$ofIndividual}   = 0;
        	$known{$ofIndividual}         = 1;
		}
  	if    ($tag eq "SEX")  { ($gender{$ofIndividual})    = $restOfLine =~ /^\s*(.*?)\s*$/; }
  	elsif ($tag eq "GIVN" and ($givenName{$ofIndividual} eq "____") ) { 
		($givenName{$ofIndividual}) = $restOfLine =~ /^\s*(.*?)\s*$/; 
	        }
  	elsif ($tag eq "SURN" and ($surName{$ofIndividual} eq "____")) { 
		($surName{$ofIndividual})   = $restOfLine =~ /^\s*(.*?)\s*$/; 
	        }
  	elsif ($tag eq "FAMC") { ($famChild{$ofIndividual})  = $restOfLine =~ /@(\S+)@/; }
  	elsif ($tag eq "FAMS") { 
		($famSpouse{$ofIndividual}[$noOfSpouses{$ofIndividual}])  = $restOfLine =~ /@(\S+)@/; 
	        $noOfSpouses{$ofIndividual}++;
	        }
  	elsif ($tag eq "SOUR" and $suppressSources != 1) { 
		($temp)  = $restOfLine =~ /@(\S+)@/;
		$duplicate = 0;
   		for ($i=1; $i<=$noOfSources{$ofIndividual}; $i++){
			if ($temp eq $sourceRef{$ofIndividual}[$i]) {$duplicate = 1}
			}
		if ($duplicate == 0) {		
			($sourceRef{$ofIndividual}[$noOfSources{$ofIndividual}])  = $restOfLine =~ /@(\S+)@/; 
	        	$noOfSources{$ofIndividual}++;
			}
	        }
	elsif ($tag eq "NOTE" and  $restOfLine =~ /\@HI/ ) {
	      # research note... do nothing
	      }
	#
	elsif (($tag eq "NOTE")  and ($restOfLine =~ /NI/)) { 
	      ($noteRefId{$ofIndividual}) = $restOfLine =~ /@(\S+)@/; 
	      }


  	if ($level eq "1") {
		$level1tag = "undefined";
		if ($tag eq "BIRT") {$level1tag = "birth"; }
		elsif ($tag eq "CHR")  {$level1tag = "chris"; }
		elsif ($tag eq "DEAT") {$level1tag = "death"; }
		elsif ($tag eq "BURI") {$level1tag = "burial"; }
		elsif ($tag eq "OCCU") {
			$level1tag = "occupation"; 
		    	$noOfOccupation{$ofIndividual}++;
		    	$typeOfOccupation{$ofIndividual}[$noOfOccupation{$ofIndividual}] = $restOfLine;
			}
		elsif ($tag eq "CENS") {
			$level1tag = "census"; 
		    	$noOfCensus{$ofIndividual}++;
		    	$relToHead{$ofIndividual}[$noOfCensus{$ofIndividual}] = $restOfLine;
			}
		}

  	elsif ($level eq "2" and $tag eq "DATE") {
		if    ($level1tag eq "birth")  {($dateOfBirth{$ofIndividual}) = $restOfLine =~ /^\s*(.*?)\s*$/; }
		elsif ($level1tag eq "chris")  {($dateOfChris{$ofIndividual}) = $restOfLine =~ /^\s*(.*?)\s*$/; }
		elsif ($level1tag eq "death")  {($dateOfDeath{$ofIndividual}) = $restOfLine =~ /^\s*(.*?)\s*$/; }
		elsif ($level1tag eq "burial") {($dateOfBurial{$ofIndividual}) = $restOfLine =~ /^\s*(.*?)\s*$/; }
		elsif ($level1tag eq "occupation") {
		    ($dateOfOccupation{$ofIndividual}[$noOfOccupation{$ofIndividual}]) = $restOfLine =~ /^\s*(.*?)\s*$/; 
		    }
	   	}
  	elsif ($level eq "2" and $tag eq "PLAC") {
		if    ($level1tag eq "birth")  {($placeOfBirth{$ofIndividual}) = $restOfLine =~ /^\s*(.*?)\s*$/; }
		elsif ($level1tag eq "chris")  {($placeOfChris{$ofIndividual}) = $restOfLine =~ /^\s*(.*?)\s*$/; }
		elsif ($level1tag eq "death")  {($placeOfDeath{$ofIndividual}) = $restOfLine =~ /^\s*(.*?)\s*$/; }
		elsif ($level1tag eq "burial") {($placeOfBurial{$ofIndividual}) = $restOfLine =~ /^\s*(.*?)\s*$/; }
		elsif ($level1tag eq "census" or "address") {
		    ($address{$ofIndividual}[$noOfAddress{$ofIndividual}]) = $restOfLine =~ /^\s*(.*?)\s*$/; 
		    $noOfAddress{$ofIndividual}++;
		    }
	   	}
 	}  #}}}2


#################################################################################
# Grab Data for Notes (NI for individuals, NF for families)
#################################################################################
#{{{2
# CONT includes a newline, CONC concatinates with no newline.


  elsif ($ident =~ /NI\S*/) {
	$noteId = $ident;
	if ($tag eq "CONT")  { ($workingString) = $restOfLine =~ /^\s*(.*?)$/;
		               $noteId{$ident} .= "br".$workingString;
		             }
	elsif ($tag eq "CONC")  { ($workingString) = $restOfLine =~ /^\s*(.*?)$/;
		               $noteId{$ident} .= $workingString;
		              }
 	}

  elsif ($ident =~ /NF\S*/) {
	$noteId = $ident;
	if ($tag eq "CONT")  { ($workingString) = $restOfLine =~ /^\s*(.*?)$/;  
		               $noteRefFam{$ident} .= "br".$workingString;
		             }
	elsif ($tag eq "CONC")  { ($workingString) = $restOfLine =~ /^\s*(.*?)$/;  
		               $noteRefFam{$ident} .= $workingString;
		             }
 	}  #}}}2


#################################################################################
# Grab Data from family Record
#################################################################################
#{{{2

  elsif ($ident =~ /F\S*/) {  # Grab data on family group. Max of 1 husband and wife, 
	                   # but zero to infinate number of children (what a thought!)
	$ofFamily = $ident;       
        if ($knownFamily{$ofFamily} != 1) { 
		$noOfKids{$ofFamily} = 0;
        	$knownFamily{$ofFamily} = 1;
		}
  	if ($tag eq "HUSB") { ($husbId{$ofFamily}) = ($restOfLine =~ /@(\S+)@/);
			      $currentSpouse = $husbId{$ofFamily};
	                     }
  	elsif ($tag eq "WIFE") { ($wifeId{$ofFamily}) = ($restOfLine =~ /@(\S+)@/);
			      $currentSpouse = $wifeId{$ofFamily};
	                     }

  	elsif ($tag eq "_PREF") { $prefSpouse{$currentSpouse}= $ofFamily }  # unique tag for Legacy

        # generate a unique string which is descriptive of the family. 
        # Use husband and wife surnames plus the unique family identifier.
        $familyFileName{$ofFamily} = $surName{$husbId{$ofFamily}}.
	                             $surName{$wifeId{$ofFamily}}.
				     $ofFamily;

  	if ($tag eq "CHIL") { # could be several children... put them into a list 
	                      # which is $noOfKids long.
	             ($id) = ($restOfLine =~ /@(\S+)@/);
		     $chilId{$ofFamily}[$noOfKids{$ofFamily}] = $id;
                     $noOfKids{$ofFamily}++;
		     # While we're here, make a note of the parents of the 
		     # individual child.
		     $fatherOf{$id} = $husbId{$ofFamily};
		     $motherOf{$id} = $wifeId{$ofFamily};
		     if ($famChild{$id} != $ofFamily) { 
			print "-E- ERROR - inconsistant information in gedcom file\n";
	     		}
		     }
  	if ($level eq "1") {
		$level1tag = "undefined";
		if ($tag eq "MARR") {$level1tag = "marriage"; }
	        elsif ($tag eq "NOTE") { ($noteRefFam{$ofFamily}) = $restOfLine =~ /@(\S+)@/ }
		}

  	elsif ($level eq "2" and $tag eq "DATE" and $level1tag eq "marriage") {
		($dateOfMarriage{$ofFamily}) = $restOfLine =~ /^\s*(.*?)\s*$/; 
	   	}
  	elsif ($level eq "2" and $tag eq "PLAC" and $level1tag eq "marriage") {
		($placeOfMarriage{$ofFamily}) = $restOfLine =~ /^\s*(.*?)\s*$/; 
	   	}
	
  	}  
      #}}}2

      
#################################################################################
# Grab Data for Sources
#################################################################################
#{{{2
# CONT includes a newline, CONC concatinates with no newline.
  if ($ident =~ /^S\S*/ and $suppressSources != 1) { 
	$ofSource = $ident;
  	if ($tag eq "ABBR")  { ($abbrSource{$ofSource}) = $restOfLine; 
		#print "-I- source ",$abbrSource{$ofSource},"\n";
	    }
  	elsif ($tag eq "TITL")  { ($titlSource{$ofSource}) = $restOfLine; 
		#print "-I- source ",$titlSource{$ofSource},"\n";
	    }
         }


 	  #}}}2

   # }}}1
   } # end of file reading.
close FH;


###################################################################
# Process the data.
###################################################################
# {{{1 
if ($disableFilter == 1) {
    print "-W- ***********************************************************\n";
    print "-W- ** WARNING - no filtering of data based on living status **\n";
    print "-W- ***********************************************************\n";
    }
else {
    filterAll();
    }

# FIXME     
foreach $indiv (@private) { 
    print "-I- Suppressing - ", $indiv, " ",$givenName{$indiv}," ",$surName{$indiv},"\n";
    $givenName{$indiv} = "private";
    $surName{$indiv} = "";
    }


    
# The Legacy tag _PREF is for the prefferred spouse in the relevent family
# so, to identify the preferred family for the individual...
#
foreach $f (keys %knownFamily) {

       # If there is no record of a spouse for either partner, assume that this 
       # is their preferreed family.

        if ($husbId{$f}) { if ($prefSpouse{$husbId{$f}}) { $prefFam{$wifeId{$f}} = $f; } }
        else { $prefFam{$wifeId{$f}} = $f; }

        if ($wifeId{$f}) { if ($prefSpouse{$wifeId{$f}}) { $prefFam{$husbId{$f}} = $f; } }
        else { $prefFam{$husbId{$f}} = $f; }
        }


# for each person, generate preferred html link.
# unmarried and no kids children have their info furthur down the page of their
# parents family.
foreach $i (keys %known) {
	if($noOfSpouses{$i} > 0 ){
            $prefFile{$i} = $familyFileName{$prefFam{$i}}.".html";
	    if ($prefFile{$i} eq ".html" ) {print "\n-E- on ",$i," ",$prefFile{$i},"\n";}
	    }
	else {
            $prefFile{$i} = $familyFileName{$famChild{$i}}.".html#".$i;
	    if ($prefFile{$i} eq (".html".$i) ) {print "\n-E- in ",$i,"\n";}
	    }
         }


	      # }}}1

###################################################################
## Remove Disjoint individual
###################################################################
disjoint();
	      
###################################################################
# Generate the family HTML files.
###################################################################
if ($suppressFamilyPages != 1) {
# {{{1
# Generate series of files, one per family.
print "-I- Writing family web pages to ",$familyDir,"\n";
$hh = 0; # reset progress indicator counter.
foreach $famId (sort keys %knownFamily) {
        $father   = $husbId{$famId};
        $mother   = $wifeId{$famId};

        # if there is a valid family.
	if ((  $known{$father} or $known{$mother}
	    ) and 
            ($filterStatus{$famId} ne "S") or $disableFilter) {

        $pgfather = $fatherOf{$father};
        $pgmother = $motherOf{$father};
        $mgfather = $fatherOf{$mother};
        $mgmother = $motherOf{$mother};

        $fileName = $familyDir . "/" . $familyFileName{$famId} . ".html";
        if (open (FAMFILE,">$fileName")) {
		#$hh = ghotiModule::progressIndicator ($hh); 
	    }
        else {print "failed to open file \"",$fileName,"\"for writting";}


###################################################
# Write HTML preamble
###################################################
# {{{2
    	print FAMFILE "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">" ;

    	print FAMFILE $optionalNewLine ;
    	print FAMFILE "<HTML><HEAD>" ;

    	print FAMFILE "<TITLE>Family of ", nameOf($father) ,,
                                    " & ", nameOf($mother), "</TITLE>";
    	print FAMFILE $optionalNewLine ;

    	print FAMFILE $optionalNewLine ;
    	print FAMFILE "<link rel=\"stylesheet\" type = \"text/css\" href = \"../".$cssFileName."\">";

    	print FAMFILE $optionalNewLine ;
    	print FAMFILE "</HEAD><body>" ;
# }}}2

###################################################
# table containing grandparents ( if they are known )
###################################################
# {{{2
        if (    $known{$pgfather} || $known{$pgmother} 
	     || $known{$mgfather} || $known{$mgmother} ) {
	print FAMFILE $optionalNewLine,"<TABLE ALIGN=\"CENTER\" WIDTH=\"95%\">"; 

	print FAMFILE "<TR>","<TD WIDTH=50%>";

# Inner table of husbands parents
        if (($known{$pgfather} || $known{$pgmother}) 
	    and $filterStatus{$pgfather} ne "L"
	    and $filterStatus{$pgmother} ne "L"
	      ){
  	  print FAMFILE $optionalNewLine,"<TABLE ALIGN=\"CENTER\" WIDTH=80% BORDER=\"2\" CELLPADDING=\"2\">"; #{{{3
  
    	  print FAMFILE $optionalNlTab;
    	  print FAMFILE "<TR><TD>";
    	  print FAMFILE "<div align=\"center\">";
	  print FAMFILE nameLinkToParents($pgfather,$father);
	  print FAMFILE yearRange ($pgfather,"<br>(",")");
    	  print FAMFILE "</div>";
    	  print FAMFILE "</TD></TR>";
  
    	  print FAMFILE $optionalNlTab;
    	  print FAMFILE "<TR><TD>";
    	  print FAMFILE "<div align=\"center\">";
	  print FAMFILE nameLinkToParents($pgmother,$father);
	  print FAMFILE yearRange ($pgmother,"<br>(",")");
    	  print FAMFILE "</div>";
    	  print FAMFILE "</TD></TR>";

    	  print FAMFILE $optionalNewLine,"</TABLE>"; #}}}3
          }

	print FAMFILE "</TD><TD WIDTH=50%>";

# Inner table of wifes parents
        if ($known{$mgfather} || $known{$mgmother}) {
    	  print FAMFILE $optionalNewLine,"<TABLE ALIGN=\"CENTER\" WIDTH=80% BORDER=\"2\" CELLPADDING=\"2\">"; #{{{3
  
    	  print FAMFILE $optionalNlTab;
    	  print FAMFILE "<TR><TD>";
    	  print FAMFILE  "<div align=\"center\">";
	  print FAMFILE nameLinkToParents($mgfather,$mother);
	  print FAMFILE yearRange ($mgfather,"<br>(",")");
    	  print FAMFILE  "</div>";
    	  print FAMFILE "</TD></TR>";
  
    	  print FAMFILE $optionalNlTab;
    	  print FAMFILE "<TR><TD>";
    	  print FAMFILE  "<div align=\"center\">";
	  print FAMFILE nameLinkToParents($mgmother,$mother);
	  print FAMFILE yearRange ($mgmother,"<br>(",")");
    	  print FAMFILE  "</div>";
    	  print FAMFILE "</TD></TR>";

    	  print FAMFILE $optionalNewLine,"</TABLE>"; #}}}3
          }

	
	print FAMFILE "</TD></TR>",$optionalNewLine,"</TABLE>"; 
	print FAMFILE  "<BR>";
        } # }}}2


###################################################
# table containing partners 
###################################################
# {{{2
	print FAMFILE $optionalNewLine,"<TABLE ALIGN=\"CENTER\" WIDTH=\"90%\" BORDER=\"4\" CELLPADDING=\"4\">";
	print FAMFILE "<TR>";

	print FAMFILE $optionalNlTab,"<TD WIDTH=50%>";
        if ($known{$father}) {
    	  print FAMFILE  "<div align=\"center\"><b>";
	  print FAMFILE nameOf($father);
	  print FAMFILE  "</b></div>";
	  }
	print FAMFILE "</TD>";

	print FAMFILE $optionalNlTab,"<TD WIDTH=50%>";
        if ($known{$mother}) {
    	  print FAMFILE  "<div align=\"center\"><b>";
	  print FAMFILE nameOf($mother);
    	  print FAMFILE  "</b></div>";
	  }
	print FAMFILE  "</TD></TR>";

	print FAMFILE  $optionalNlTab,"<TR><TD valign=\"top\">";
        if ($known{$father}) {
          print FAMFILE allAboutId($father,$famId);
	  }
	print FAMFILE  "</TD>";

	print FAMFILE  $optionalNlTab,"<TD valign=\"top\">";
        if ($known{$mother}) {
          print FAMFILE allAboutId($mother,$famId);
	  }
	print FAMFILE  "</TD></TR>";

	print FAMFILE $optionalNewLine,"</TABLE>";  # }}}2

###################################################
# table containing marriage and notes
###################################################
# {{{2
        $tempString = allAboutFam ($famId);
           if ($tempString ne "") {
	   print FAMFILE $optionalNewLine,"<TABLE ALIGN=\"CENTER\" WIDTH=\"90%\" BORDER=\"4\" CELLPADDING=\"4\">";
	   print FAMFILE "<TR>";
	   print FAMFILE "<TD>";
           print FAMFILE allAboutFam ($famId);
	   print FAMFILE "</TD>";
	   print FAMFILE "</TR>";
	   print FAMFILE $optionalNewLine,"</TABLE>";  # }}}2
           }

 	print FAMFILE "<div align=\"center\">";

###################################################
## list Children in the family
###################################################
        if ($noOfKids{$famId} > 0 ){
# {{{2
#if f.children then
	$noBoys{$famId}=0;
	$noGirls{$famId}=0;
	$noGenderUnknown{$famId}=0;
	$noLiving{$famId}=0;
	foreach ($i=0; $i<$noOfKids{$famId}; $i++) {
		if    ( $gender{ $chilId{$famId}[$i]} eq "F" ) {$noGirls{$famId} += 1}
		elsif ( $gender{ $chilId{$famId}[$i]} eq "M" ) {$noBoys{$famId} += 1}
		elsif ( $gender{ $chilId{$famId}[$i]} eq "U" ) {$noGenderUnknown{$famId} += 1} 

		if ($filterStatus{$chilId{$famId}[$i]} eq "L") {$noLiving{$famId} += 1}
		}
 
 	print FAMFILE $optionalNewLine,"<b>Children:</b>";
 	print FAMFILE $optionalNewLine,"<table align=\"center\" width=50% border=\"2\" cellpadding=\"2\">";
 	print FAMFILE "<tr><td>";
 	print FAMFILE "<div align=\"center\">";

	if (($noBoys{$famId}+$noGirls{$famId}+$noGenderUnknown{$famId}) > 0) {
		print FAMFILE "They had ";
		print FAMFILE ghotiModule::sentanceList(
	                $noBoys{$famId}," boy","boys",
			$noGirls{$famId},"girl","girls",
	                $noGenderUnknown{$famId},"child of unknown sex","children of unknown sex");
		}
		#print FAMFILE $noBoys{famId}," " ,$noGirls{famId}," " ,$noGenderUnknown{famId}," ";
		print FAMFILE "<br>";
       if ($noLiving{$famId} > 1) {
	       print FAMFILE "<small>(" . $noLiving{$famId} . 
	                     " children may be alive or have a living spouse)<BR></small>"
		     }
       elsif ($noLiving{$famId} == 1) {
	       print FAMFILE "<small>(", $noLiving{$famId}, 
	                     " child may be alive or has a living spouse)<BR></small>"
		     }


        # once the summary message has been printed, if any child is alive, reset the number of children.
#        if ($filterStatus{$famId} ne "R" or $disableFilter){

	   foreach ($i=0; $i<$noOfKids{$famId}; $i++) {
	     $temp = $chilId{$famId}[$i] ;
	     if ($filterStatus{$temp} ne "L") {
	     	print FAMFILE $optionalNlTab, nameLinkToChild($temp), yearRange ($temp," (",")"),"<br>";
	          }
	     }

           # list 1/2 brothers and sisters through father
           foreach ($f=0; $f < $noOfSpouses{$father}; $f++) {
	  	if ($famSpouse{$father}[$f] ne $famId) {
			foreach ($j=0; $j<$noOfKids{$famSpouse{$father}[$f]}; $j++) {
				$temp = $chilId{$famSpouse{$father}[$f]}[$j] ;
	     			if ($filterStatus{$temp} ne "L") {
	  			  print FAMFILE $optionalNlTab,"1/2 ",nameLinkToChild($temp), 
				                yearRange ($temp," (",")"),"<br>";
          			  }
          			}
          		}
          	}

           # list 1/2 brothers and sisters through mother
           foreach ($f=0; $f < $noOfSpouses{$mother}; $f++) {
	  	if ($famSpouse{$mother}[$f] ne $famId) {
			foreach ($j=0; $j<$noOfKids{$famSpouse{$mother}[$f]}; $j++) {
				$temp = $chilId{$famSpouse{$mother}[$f]}[$j] ;
	     		        if ($filterStatus{$temp} ne "L") {
	  			  print FAMFILE $optionalNlTab,"1/2 ",nameLinkToChild($temp),
				                yearRange ($temp," (",")"),"<br>";
          			  }
          			}
          		}
          	}
#   	}

 	print FAMFILE "</div>";
 	print FAMFILE "</TD></TR>",$optionalNewLine,"</TABLE>"; 

#}}}2
        }
print FAMFILE "<BR>"x2, "<HR>", $footnoteLine;



###################################################
## print info on Children in the family if they have
## no known family of their own.
###################################################
#if ($filterStatus{$famId} ne "R"){ # {{{2

	#for c in f.children do  # for every child
	foreach ($l=0; $l<$noOfKids{$famId}; $l++) {
		#if ~c.marriages then  # if the child has no family
		if ($noOfSpouses{$chilId{$famId}[$l]}==0) {
			print FAMFILE "<br>" x 20;
	  		$temp = $chilId{$famId}[$l] ;
      			print FAMFILE "<A NAME=\"",$temp, "\"></A>" ;
      			print FAMFILE $optionalNewLine,"<TABLE WIDTH=40% BORDER=\"2\" CELLPADDING=\"2\">";
      			print FAMFILE "<TR><TD>";
    	  		print FAMFILE "<div align=\"center\">";
	  		print FAMFILE nameLinkToParents($father,$temp);
	  		print FAMFILE yearRange ($father,"<br>(",")");
      			print FAMFILE "</div>";
      			print FAMFILE "</TD></TR>";
      			print FAMFILE "<TR><TD>";
    	  		print FAMFILE "<div align=\"center\">";
	  		print FAMFILE nameLinkToParents($mother,$temp);
	  		print FAMFILE yearRange ($mother,"<br>(",")");
      			print FAMFILE "</div>";
      			print FAMFILE "</TD></TR>";
    			print FAMFILE $optionalNewLine,"</TABLE><BR>";



###################################################
## PRINT unmarried child Table header
###################################################
#{{{3


      			print FAMFILE $optionalNewLine,"<TABLE WIDTH=60% BORDER=\"4\" CELLPADDING=\"4\">";
      			print FAMFILE "<TR><TD>";
      			print FAMFILE "<div align=\"center\"><b>";
      			print FAMFILE nameOf($temp);
      			print FAMFILE "</b></div>";
      			print FAMFILE "</TD></TR>"; # }}}3


###################################################
# PRINT unmarried child Table information
###################################################
#{{{3
     			print FAMFILE "<TR><TD>";
     			print FAMFILE allAboutId($temp,0);
     			print FAMFILE "</TD></TR>";
     			print FAMFILE $optionalNewLine,"</TABLE>"; # }}}3
			print FAMFILE "<BR>"x2, "<HR>", $footnoteLine;
 			}
 		} # }}}2
	#}
        print FAMFILE "<br>" x 20;
 	print FAMFILE "</div>";
 	print FAMFILE "</html></body>";
        close FAMFILE;
        }  # end if valid family data
   } # end for each family

  # }}}1
}

###################################################################
# Create Index file.
###################################################################
if ($supressIndexGen != 1) {
# {{{1

print "-I- Compiling surname index..\n";
$hh = 0; # reset progress indicator counter.

# add-up the number of each surname. {{{2  
foreach $i (keys %surName) {
   $surCount{$surName{$i}}++;
   if ($filterStatus{$i} eq "L") {
   	$livingCount{$surName{$i}}++;
	}
   }
  # }}}2

# Set up fixed strings {{{2
$indexFileTop = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">"
              . "<html><head>"
              . "<TITLE>Index of ". $targetDir. " Family History</TITLE>"
              . "<link rel=\"stylesheet\" type = \"text/css\" href = \"".$cssFileName."\">"
              . "</head><body>". $optionalNewLine
              . "<table align=\"center\" width=80% border=\"2\" cellpadding=\"2\">". $optionalNewLine;

$indexFileBottom = "</table></html></body>";

$indexMenuTop = "<HTML><HEAD><TITLE>MainMenu</TITLE>"
              . "<link rel=\"stylesheet\" type = \"text/css\" href = \"".$cssFileName."\">"
	      . "</HEAD><body><p><A HREF=readme.html target=\"Body\">Overview</A></p>";
$indexMenuBottom = "</BODY></HTML>";
  # }}}2


open (INDEXMENU,">".$targetDir."/menu.html");
print INDEXMENU $indexMenuTop;

$indexPage = 1;
open (INDEXFILE,">".$targetDir."/contents".$indexPage.".html");
print INDEXFILE $indexFileTop;

if ($suppressSources != 1) { print INDEXMENU "<p><A HREF=sources.html target=\"Body\">Sources</a></p>"; }

$personsOnThisPage = 0;
foreach $sname (sort keys %surCount) {

   if ( $personsOnThisPage > $minPersonsPerIndexPage) {  # close the current index page, generate a link to it and open the next one. {{{2
              $personsOnThisPage = 0; 

              print INDEXMENU "<p><A HREF=contents",$indexPage,".html target=\"Body\">";
	      if ($firstSnameOnPage eq $previousSname) { print INDEXMENU $firstSnameOnPage,"</a></p>"; }
	      else { print INDEXMENU $firstSnameOnPage," to ",$previousSname,"</a></p>"; }

	      print INDEXFILE $indexFileBottom;
	      close INDEXFILE;
	      $indexPage++;
	      open (INDEXFILE,">".$targetDir."/contents".$indexPage.".html");
	      print INDEXFILE $indexFileTop;
 	} # }}}2

   if ($sname ne "" and $surCount{$sname}!=0) { 
	   #$hh = ghotiModule::progressIndicator ($hh); 
        print INDEXFILE "<tr><td valign=\"top\">", $optionalNlTab;
	print INDEXFILE "<b>", uc $sname, "</b> (", $surCount{$sname},")</td><td>";
        if ($livingCount{$sname}) {
                if ($livingCount{$sname} == 1) {
			print INDEXFILE $livingCount{$sname}," person is living or has a living spouse<br>"; }
         	else {
			print INDEXFILE $livingCount{$sname}," people are living or have a living spouse<br>";
			}
		}

        # Living people are suppressed, Historic or Ancient individual have links, 
        # Recent Dead people have name only. Nb. Theses may have a living sibling. 
	foreach $i (sort sortByGivenName keys %givenName) {
		if ($sname eq $surName{$i}) {
                        if ( $personsOnThisPage eq 0 ) {$firstSnameOnPage = $sname}
			if ($filterStatus{$i} eq "L" ){ 
                            if ( $personsOnThisPage eq 0 ) {$personsOnThisPage++;}
			    }
			elsif ($filterStatus{$i} eq "D" 
			   and ($filterStatus{$fatherOf{$i}} eq "L"
				   or $filterStatus{$motherOf{$i}} eq "L")){ 
		   	    print INDEXFILE $givenName{$i},
		   	          yearRange ($i," (",")"),"<BR>", $optionalNewLine;
		   	    $personsOnThisPage++;
			    print " -W- deceased with living parent ",$givenName{$i}," ",$surName{$i},
			           yearRange ($i," (",")"),"\n"; 
			    }
			else {
			    print INDEXFILE nameFromIndex($i),
			          yearRange($i," (",")") ,"<BR>", $optionalNewLine;
                            $personsOnThisPage++;
			    }
        		}
        	}
	print INDEXFILE "</TD></TR>";
        }
	$previousSname=$sname;
   }

print INDEXMENU "<p><A HREF=contents",$indexPage,".html target=\"Body\">",$firstSnameOnPage," to ",$previousSname,"</A></p><br><br>" ;
print INDEXMENU $indexMenuBottom;
close INDEXMENU;	

print INDEXFILE "</table>" ;
print INDEXFILE $indexFileBottom;
close INDEXFILE;	

print "..completed\n";

#}}}1
}

###################################################################
# Create Source file.
###################################################################
if ($supressIndexGen != 1) {
# {{{1


#$hh = 0; # reset progress indicator counter.


# Set up fixed strings {{{2
$sourceFileTop = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">"
              . "<html><head>"
              . "<TITLE>Sources of ". $targetDir. " Family History</TITLE>"
              . "<link rel=\"stylesheet\" type = \"text/css\" href = \"".$cssFileName."\">"
              . "</head><body>". $optionalNewLine
              . "<table align=\"center\" width=80% border=\"2\" cellpadding=\"2\">". $optionalNewLine;

$sourceFileBottom = "</table></body></html>";

  # }}}2


if ($suppressSources != 1){ 
	print "-I- compiling source index..";
	open (SOURCELIST,">".$targetDir."/sources.html");
	print SOURCELIST $sourceFileTop;


	foreach $s ( sort sortBySourceName keys %abbrSource) {
		print SOURCELIST  "<tr><td>",$abbrSource{$s},"</td>" .
	                      	"<td>",$titlSource{$s}."</td></tr>" . $optionalNewLine;
	}
	
	print SOURCELIST $sourceFileBottom;
	close SOURCELIST;	

	print "..completed\n";
	}

#}}}1
}


if ($suppressExistingLinkChecks != 1) {
# {{{1

   print "-I- Checking links..";
   $failCount = 0;

   open (REPORTFILE,">".$ghotiLinkReport);
	print REPORTFILE "List of links\n";
	print REPORTFILE "~~~~~~~~~~~~~\n";
   close REPORTFILE;

   for ($i=1; $i<=$indexPage; $i++){
      	$failCount += ghotiModule::checkLinks($targetDir,("contents".$i.".html"), $ghotiLinkReport) ;
	}
   print "..completed\n";
   if ($failCount > 0) {
	print "-E- ",$failCount, " links are broken\n";
   	ghotiModule::examineLinkReport($ghotiLinkReport) ;
	}
	#}}}1
  }

###################################################################
# End of execution
###################################################################


###################################################################
# Subroutines that reference global data.
###################################################################
#{{{1

###################################################################
# sort selector routines which may be used to sort the index page.
sub sortByGivenName { $givenName{$a} cmp $givenName{$b}; }

sub sortBySourceName { lc $abbrSource{$a} cmp  lc $abbrSource{$b}; }
#sub sortBySourceName { (lc $abbrSource{$a}) cmp (lc $abbrSource{$b}); }

sub sortByYearOfBirth { #{{{2
    my $yearOfBirthA = "";
    my $yearOfBirthB = "";
    ($yearOfBirthA) = estBirth($a) =~ /(\d\d\d\d)\s*$/; 
    ($yearOfBirthB) = estBirth($b) =~ /(\d\d\d\d)\s*$/; 
    $yearOfBirthA <=> $yearOfBirthB; } #}}}2
    

sub allAboutId {    #{{{2
################################################################## 
    my $ident = $_[0];

    if ($privatePerson{$ident} eq "Blocking") {
    	$returnString = "Sorry - information on this person is restricted to close family members.";
    	}
    else	{

    	my $currentFamily = $_[1];
    	my $returnString = "";
    	if ($dateOfBirth{$ident}) { 
		$returnString .= "<b>born: </b>" . $dateOfBirth{$ident} ;
		}

    	if ($dateOfChris{$ident}) { 
		$returnString .= "<br><b>christened: </b>" . $dateOfChris{$ident} ;
		}


    	if ($dateOfDeath{$ident}) { 
		$returnString .= "<br><b>died: </b>" . $dateOfDeath{$ident};
		}
	    
    	if ($dateOfBurial{$ident}) { 
		$returnString .= "<br><b>burial: </b>" . $dateOfBurial{$ident};
		}

    	if ($noOfSpouses{$ident} > 1 ) { 
		$returnString .= "<br><b>spouses: </b>" . linkToSpouses($ident,$currentFamily) ;
		}


    	if ($noteRefId{$ident}) {
		$returnString .= "<br>" .
	        	ghotiModule::convertRefsToHTML(
	                 	ghotiModule::convertNoteToHTML($noteId{$noteRefId{$ident}}),
                                                        	$targetDir    ,
                                                        	$bigPicRoot   ,
                                                        	$smallPicRoot ,
                                                        	$htmlRoot     ,
								$masterDir
					      	);
			}


    	if ($noOfOccupation{$ident}) {
	   $returnString .= "<br><small><u>$noOfOccupation{$ident} occupation";
	   if ($noOfOccupation{$ident} == 1 ){ $returnString .= "</u>"}
	   else                      { $returnString .= "s</u>"}

    	   foreach ($i=1; $i<$noOfOccupation{$ident}+1; $i++) {
		$returnString .= "<br>".$typeOfOccupation{$ident}[$i]." - ".$dateOfOccupation{$ident}[$i];
		}
	   $returnString .= "</small>";
	   }


    	if ($noOfSources{$ident}) { 
	   $returnString .= "<br><small><u>$noOfSources{$ident} source";
	   if ($noOfSources{$ident} == 1 ){ $returnString .= "</u><br>"}
	   else                      { $returnString .= "s</u><br>"}

    	   foreach ($i=0; $i<$noOfSources{$ident}; $i++) {
		  $returnString .= " [". $abbrSource{$sourceRef{$ident}[$i]} . "]" ;
		  }
	   $returnString .= "</small>";
           }


    	return ($returnString);
	}
} #}}}2


sub allAboutFam {    #{{{2
###################################################################
    my $famId = $_[0];
    my $returnString = "";
    if ($dateOfMarriage{$famId}) { 
	$returnString .= "<b>married on: </b>" . $dateOfMarriage{$famId} . " ";
	}
    if ($placeOfMarriage{$famId}) { 
	$returnString .= "<b>at </b>" . $placeOfMarriage{$famId} . "<br>";
	}
    if ($noteRefFam{"N".$famId}) { 
	$returnString .= "<br>" 
	               . ghotiModule::convertRefsToHTML(
	                    ghotiModule::convertNoteToHTML($noteRefFam{"N".$famId}),
                                                        $targetDir    ,
                                                        $bigPicRoot   ,
                                                        $smallPicRoot ,
                                                        $htmlRoot     ,
							$masterDir
				         	      );
	# ghotiModule::convertNoteToHTML($noteFam{$noteRefFam{$famId}}); 
	}
    return ($returnString);

} #}}}2


sub nameOf { #{{{2
###################################################################
    my $ident = $_[0];
    if ($givenName{$ident} || $surName{$ident}) {
	  return ($givenName{$ident}." ".uc $surName{$ident});
          }
    else  {
          return ("unknown");
          }
} #}}}2


sub yearRange {    #{{{2
###################################################################
# Returns a string of the form "1890 - 1932"
# There are 2 optional arguments. If a valid date range is found argument 1 is 
# prepended and argument 2 is appended
# If no valid year values are found, return empty string. 
    my $ident = $_[0];
    my $startString = $_[1];
    my $endString = $_[2];
    my $yearOfBirth = "";
    my $yearOfDeath = "";

    ($yearOfBirth) = estBirth($ident) =~ /(\d\d\d\d)\s*$/; 
    ($yearOfDeath) = $dateOfDeath{$ident} =~ /(\d\d\d\d)\s*$/; 

    if ($yearOfBirth == "" && $yearOfDeath == "") {
    	return ("");
       }
    else { return (  $startString . $yearOfBirth .  " - " . $yearOfDeath . $endString);
       }
}   #}}}2


sub nameLinkToParents {    #{{{2
###################################################################
    my $ident = $_[0];
    my $idChild = $_[1];
    if ($givenName{$ident} || $surName{$ident}) {
	    #if ($filterStatus{$ident} eq "L") { return ("living"); }  FIXME reove these comments
	    #else {
	  	return ("<b><a href=\"".$familyFileName{$famChild{$idChild}}.".html\">".
			$givenName{$ident}." ".uc $surName{$ident}.
			"</a></b>");
		#	}
          }
    else  {
          return ("unknown");
          }
}   #}}}2


sub nameLinkToChild { # {{{2
###################################################################
# The only difference between this and nameFromindex is "families/" in the path
    my $ident = $_[0];
    	if ($givenName{$ident} || $surName{$ident}) {

	    if ($filterStatus{$prefFam{$ident}} eq "S" ){ # and not($disableFilter) ) {
                # should be unexecutable
                print "\n-E- ANCIENT OR HISTORIC PERSON IN SUPPRESSED FAMILY "; 
		$temp = $prefFam{$ident};
		print $givenName{$husbId{$temp}}," ", $surName{$husbId{$temp}},$filterStatus{$husbId{$temp}} , 
		" - ", $givenName{$wifeId{$temp}}," ", $surName{$wifeId{$temp}};
	    	return ( $givenName{$ident}." ".uc $surName{$ident});
		}
            else {
	    	return ("<b><a href=\"".$prefFile{$ident}."\">".
		    $givenName{$ident}." ".uc $surName{$ident}.
		    "</a></b>");
          	}
            }
    	else  {
	  	return ("unknown");
            }
} #}}}2


sub nameFromIndex {  #{{{2
###################################################################
# The only difference between this and nameLinkToChild is "families/" in the path
    my $ident = $_[0];
      if ($givenName{$ident} || $surName{$ident}) {
	      
	    if ($filterStatus{$prefFam{$ident}} eq "S") {
                # should be unexecutable
                print "\n-E- ANCIENT OR HISTORIC PERSON IN SUPPRESSED FAMILY "; 
		print $givenName{$ident}," ", $surName{$ident}, "\n";
	    	return ( $givenName{$ident}." "); #uc $surName{$ident});
		}
            else {
	        return ("<b><a href=\"families/".$prefFile{$ident}."\">".
		        $givenName{$ident}." ".   #uc $surName{$ident}.
		        "</a></b>");
          	}
        }
      else  {
        return ("unknown");
        }
} #}}}2


sub linkToSpouses {   #{{{2
###################################################################
# Returns a string similar to "1, 2, 3" with links to the families. 
# Except for the current Family, for which no link is generated
# because you're already there!
    my $ident = $_[0];
    my $currentFamily = $_[1]; 
    my $returnString = "";

    if ($noOfSpouses{$ident} > 1) {
	$returnString .= "<br>";
	foreach ($spouseCount=0; $spouseCount < $noOfSpouses{$ident}; $spouseCount++){
		if ($spouseCount != 0) {$returnString .= ", "}
                        $tempFam = $famSpouse{$ident}[$spouseCount];
			if ($tempFam eq $currentFamily) { 
				$returnString .= ($spouseCount +1);
				}
                        else {
	  			$returnString .= "<a href=\"".$familyFileName{$tempFam}.".html\">" .($spouseCount + 1) ."</a>";
				}
		}
	}
    return ($returnString)
}  #}}}2


sub filterAll {  #{{{2

print "-I- Applying filteration algorithm"; 

# Historic means whole of this generation is deceased
# Ancient is older then Historic
# Dead is for known dead who are not Historic or Ancient.

print "-I- 1. Identify Ancient and Historic individuals\n";
# if born > 150 years ago or died > 100 years ago, set as Ancient 
# if born between 100 and 150 years ago, classify as Historic.

foreach $person (keys %known) {
		if( ghotiModule::gtXYearsAgo(estBirth($person),150) == 1 or
		    ghotiModule::gtXYearsAgo($dateOfDeath{$person},100) ==1 ) {
			$filterStatus{$person} = "A";
			if ( $surName{$person} eq "KNIGHT")  {print "setting $dateOfBirth{$person} $givenName{$person} $givenName{$person} $surName{$person} to A\n";}
			#$hh = ghotiModule::progressIndicator ($hh); 
			}
		elsif (ghotiModule::gtXYearsAgo(estBirth($person),100) == 1) {
			$filterStatus{$person} = "H";
			if ( $surName{$person} eq "KNIGHT")  {print "setting $dateOfBirth{$person} $givenName{$person} $surName{$person} to H\n";}
			#$hh = ghotiModule::progressIndicator ($hh); 
			}
		else {  
			$filterStatus{$person} = "U";
			if ( $surName{$person} eq "KNIGHT")  {print "setting $givenName{$person} $surName{$person} to U\n";}
			}
		}

print "-I- 2. Derive Ancient and Historic status based on parents and child status\n";
# If either parent is ancient and child is not assigned, child must be historical
# If any child is historical, individual is ancient.
# repeat this process until stable.
		

$continueProcessing = 1;
while ($continueProcessing==1) {
    $continueProcessing = 0;
     
    foreach $person (keys %known) {

        # if either parent is Ancient, child is Historic, unless that child is likely to be living.
	   if ( ($filterStatus{$person} eq "U") and 
		($filterStatus{$fatherOf{$person}} eq "A" or
	         $filterStatus{$motherOf{$person}} eq "A") ) {
	 if ((estBirth($person) eq 0) or ( ghotiModule::gtXYearsAgo(estBirth($person),100)==1 ) or
	     ($dateOfDeath{$person} ne "") ) { 
				$filterStatus{$person} = "H";
				$diag = 0;
	 if (estBirth($person) eq 0) {$diag += 1;};
	 if ( ghotiModule::gtXYearsAgo(estBirth($person),100)==1 ) {$diag += 2};
	 if ($dateOfDeath{$person} ne "") {$diag +=4}; 
				print "  H$diag - $givenName{$person} $surName{$person} ", estBirth($person)," - $dateOfDeath{$person}\n";
                        	$continueProcessing = 1; 
			}
		}

        # if any child in a family is H or A, then any parents are A.
	    if ( $filterStatus{$person} eq "H" or
	         $filterStatus{$person} eq "A" )  {
				if ($known{$fatherOf{$person}}) {
	                        	$i = $fatherOf{$person};
					if ($filterStatus{$i} ne "A"){
						$filterStatus{$i} = "A";
						$continueProcessing = 1;
						}
					}
				if ($known{$motherOf{$person}}) {
	                        	$i = $motherOf{$person};
					if ($filterStatus{$i} ne "A"){
						$filterStatus{$i} = "A";
						$continueProcessing = 1;
						}
					}
	       }
	#$hh = ghotiModule::progressIndicator ($hh); 
    }
    }



print "-I- 3. Identify Historic individuals based on spouse(s)\n";
# for each family, if one partner is historic or ancient then the other one must be 
$continueProcessing = 1;
while ($continueProcessing==1) {
    $continueProcessing = 0;
     
   foreach $famId (sort keys %knownFamily) {
      if (  $filterStatus{$wifeId{$famId}} eq "U" and
            ( $filterStatus{$husbId{$famId}} eq "H" or 
              $filterStatus{$husbId{$famId}} eq "A" )) { 
                    $filterStatus{$wifeId{$famId}} = $filterStatus{$husbId{$famId}}; 
                    $continueProcessing = 1;
		    }
      if (  $filterStatus{$husbId{$famId}} eq "U" and
            ( $filterStatus{$wifeId{$famId}} eq "H" or 
              $filterStatus{$wifeId{$famId}} eq "A" )) { 
		    $filterStatus{$husbId{$famId}} = $filterStatus{$wifeId{$famId}};
                    $continueProcessing = 1;
		    }
      }
   }


print "-I- 4. Identify Deceased individuals\n";
$hh = 0; # reset progress indicator counter.
# if a preson is dead, and they are not historical or ancient, classify as "D"
foreach $person (keys %known) {
		if ($filterStatus{$person} ne "A" and $filterStatus{$person} ne "H") {
			if( $dateOfDeath{$person} ) {
				$filterStatus{$person} = "D";
                                }
			elsif ((estBirth($person) != 0) and (ghotiModule::gtXYearsAgo(estBirth($person),100)==0)) {
				$filterStatus{$person} = "L";
				}
			}
		}



print "-I- 5. Assume everyone else is living\n";
# for each person, if not dead, historic or ancient, assume they are Living.
$hh = 0; # reset progress indicator counter.
foreach $person (keys %known) {
			if ($filterStatus{$person} ne "A" and 
                            $filterStatus{$person} ne "H" and
                            $filterStatus{$person} ne "D") {
			    	#$hh = ghotiModule::progressIndicator ($hh); 
				$filterStatus{$person} = "L";
				}
		}


# however, if a dead person has a living spouse... define as living for purposes of privacy
print "-I- 6. deceased spouse of living individuals are treated as living to respect the privacy of the living spouse\n";
foreach $person (keys %known) {
	if ($filterStatus{$person} eq "D" ) {
	    foreach ($f=0; $f < $noOfSpouses{$person}; $f++) {
		if ($filterStatus{$husbId{$famSpouse{$person}[$f]}} eq "L" ) {
		    $filterStatus{$person} = "L";
		    print "\t",$givenName{$person}," ",$surName{$person}," has husband who may be alive.\n";
		    }
		elsif ($filterStatus{$wifeId{$famSpouse{$person}[$f]}} eq "L" ) {
		    $filterStatus{$person} = "L";
		    print "\t",$givenName{$person}," ",$surName{$person}," has wife who may be alive.\n";
    }	}	}   }


 


#print "\n 7. Suppress family if either parent is alive. Restrict information if any child is alive\n";
# for each family
# status S = Suppressed
# status R = Restricted (print number of boys and girls only)
foreach $famId (sort keys %knownFamily) {
        $filterStatus{$famId} = "U";
	if ($filterStatus{$husbId{$famId}} eq "L" or 
            $filterStatus{$wifeId{$famId}} eq "L" ) {
               $filterStatus{$famId} = "S"; # suppressed
              }
        else {
#	foreach ($i=0; $i<$noOfKids{$famId}; $i++) { 
#		if    ( $filterStatus{ $chilId{$famId}[$i]} eq "L" ) {
#                   $filterStatus{$famId} = "R"; 
#                   $hh = ghotiModule::progressIndicator ($hh); 

#		   $father   = $husbId{$famId};
#		   $mother   = $wifeId{$famId};

                   # set any step families to restricted as well
#                   foreach ($f=0; $f < $noOfSpouses{$father}; $f++) {
#	  	         if ($famSpouse{$father}[$f] ne $famId and
#                             $filterStatus{$famSpouse{$father}[$f]} ne "S") { 
#                               $filterStatus{$famSpouse{$father}[$f]} = "R"; 
#			       $hh = ghotiModule::progressIndicator ($hh); 
# 			    }
#                         }

                   # set any step families to restricted as well
#                   foreach ($f=0; $f < $noOfSpouses{$mother}; $f++) {
#	  	         if ($famSpouse{$mother}[$f] ne $famId and
#                             $filterStatus{$famSpouse{$mother}[$f]} ne "S") { 
#                               $filterStatus{$famSpouse{$mother}[$f]} = "R"; 
#			       $hh = ghotiModule::progressIndicator ($hh); 
# 			    }
#                         }
#			 #
#               #print $familyFileName{$famId}, "is restricted\n";
#                   last;
#                       
#		   }
#		} # end of for each child
       }
    } # end of for each family

print "-I- filter algorithm completed\n";

    
} # }}}2


sub estBirth {  #{{{2
    my $i = $_[0];

    	if ($dateOfBirth{$i}) { return( $dateOfBirth{$i} ) }
	elsif ($dateOfChris{$i}) {
    		($year) = $dateOfChris{$i} =~ /(\d\d\d\d)\s*$/; 
		return ( $year ) }
     	else {return (0) }

} # }}}2



sub estDeath {  #{{{2
    my $i = $_[0];

    	if ($dateOfDeath{$i}) { return( $dateOfDeath{$i} ) }
	elsif ($dateOfBurial{$i}) {
    		($year) = $dateOfBurial{$i} =~ /(\d\d\d\d)\s*$/; 
		return ( $year ) }
     	else {return(0)}

} # }}}2


sub disjoint {  #{{{2

print "-I- Compiling list of individuals related to " 
    . $givenName{$seedId}." ".$surName{$seedId},"\n"; 
 

foreach $person (keys %known) { $disjoint{$person} = 1; }


$seedId = "I1";
	
$disjoint{$seedId} = 0;

open (SEEDREPORT,">seedReport.txt");

$continueProcessing = 1;
while ($continueProcessing==1) {
    $continueProcessing = 0;
     
    foreach $person (keys %known) {

        # if either parent is Joined, child is Joined.
	   if ( $disjoint{$person} == 1) {
		if ($known{$fatherOf{$person}} and $disjoint{$fatherOf{$person}} == 0  and
				              $privatePerson{$fatherOf{$person}} ne "Blocking") {
			$disjoint{$person} = 0;
			print SEEDREPORT "$person set as child of father $fatherOf{$person}\n";
                        $continueProcessing = 1; 
			}
		elsif ($known{$motherOf{$person}} and $disjoint{$motherOf{$person}} == 0 and
				                 $privatePerson{$motherOf{$person}} ne "Blocking") {
			$disjoint{$person} = 0;
			print SEEDREPORT "$person set as child of mother $motherOf{$person}\n";
                        $continueProcessing = 1; 
			}
		}

        # if any child in a family is joined, then both parents are joined.
	    if ( $disjoint{$person} == 0 and $privatePerson{$person} ne "Blocking")  {
	        $i = $fatherOf{$person};
		if ($known{$i}) {
			if ($disjoint{$i} == 1){
				$disjoint{$i} = 0;
				print SEEDREPORT "$i set as father of $person\n";
				$continueProcessing = 1;
				}
			}
	        $i = $motherOf{$person};
		if ($known{$i}) {
			if ($disjoint{$i} == 1){
				$disjoint{$i} = 0;
				print SEEDREPORT "$i set as mother of $person\n";
				$continueProcessing = 1;
				}
			}
		}
	}

	foreach $famId (sort keys %knownFamily) {
      	  $wf = $wifeId{$famId};
      	  $hb = $husbId{$famId};
      	  if (  $disjoint{$wf} == 0 and $privatePerson{$wf} ne "Blocking" and $disjoint{$hb} == 1) {
       	 	print SEEDREPORT "$hb set as husband of $wf\n";
	 	$disjoint{$hb} = 0;
	 	$continueProcessing = 1;
	 	}

      	  if (  $disjoint{$hb} == 0 and $privatePerson{$hb} ne "Blocking" and $disjoint{$wf} == 1) {
       	 	print SEEDREPORT "$wf set as wife of $hb\n";
	 	$disjoint{$wf} = 0;
	 	$continueProcessing = 1;
	 	}
	
      	   }
    }
print SEEDREPORT "-I- Deleting Disjoint Individuals\n";

$k=0;
foreach $i (keys %known) {
	if ($disjoint{$i} == 1 ) { 
		print SEEDREPORT "\t",$givenName{$i}," ",$surName{$i},"  ",$i,"\n",; 
                --$surCount{$surName{$i}} ; ++$k;
		delete $known{$i}; delete $surname{$i}; delete $givenName{$i}; 
		}
	}
print "-I- Deleted $k Disjoint Individuals\n";


$temp = 0;
$k=0;
foreach $famId (sort keys %knownFamily) {
        $father   = $husbId{$famId};
        $mother   = $wifeId{$famId};

        # if there is a valid family.
	if (  ($known{$father} and $privatePerson{$father} ne "Blocking") or 
	       ($known{$mother} and $privatePerson{$mother} ne "Blocking")  ) {
		}	
	    else {
            $filterStatus{$famId} = "S" ;
	    if ($temp == 0) { print SEEDREPORT "-I- Suppressing families:"; $temp=1;}
	    print SEEDREPORT " ",$famId;
	    ++$k;
            }
        }

print "-I- Deleted $k families which have neither partner identifyable\n";


close SEEDREPORT;

} # }}}2

#}}}1

__END__


